; ================================================================= ;
; PICAXE Software by Ben Ryves 2007                                 ;
; ================================================================= ;
; To handle communications between the TI-83 Plus (with its limited ;
; serial I/O link port) and more complex add-ons, an extra piece of ;
; hardware has to be put between the two.                           ;
; In this case, the excellent PICAXE-28X1 is a suitable component   ;
; given its low cost, large memory, and extensive built-in support  ;
; for an array of exciting hardware communication protocols.        ;
; ================================================================= ;


; ================================================================= ;
; Device Configuration.                                             ;
; ================================================================= ;
{
#PicAxe 28X1 ; Designed for the 28X1 series.
#Terminal 9600
;#Define Debug
}

; ================================================================= ;
; Variable Allocation
; ================================================================= ;
{
Symbol TiLPInputBuffer = B10
}


; ================================================================= ;
; Entry Point / Main Loop.                                          ;
; ================================================================= ;
{
Main:
	
	; Use 8MHz clock speed.
	SetFreq M8
	
	; TiLP input buffer pointer:
	TiLPInputBuffer = 0

	; Set up background serial receive.
	HSerSetup B9600_8, %01

	; RTS
	Low 0
	
	Do
		; Try and read a byte from the calculator.
		GoSub TiLPIn

		; B1 = received bits - have we received 8?
		If B1 = 8 Then
		
			#IfDef Debug
				SerTxd("In", 9, B0, 9, #B0, CR, LF)
			#EndIf		
		
			; Store the byte in EEPROM (buffer).
			Write TiLPInputBuffer, B0
			
			If B0 = "\r" Or TiLPInputBuffer = 255 Then
				; If it's a carriage return, output the buffer.
				For B2 = 0 To TiLPInputBuffer
					; Read from buffer and output.
					Read B2, B0 : HSerOut 0, (B0)
				Next B2
				; Reset buffer pointer.
				TiLPInputBuffer = 0
			Else
				; Not a carriage return, so move on.
				Inc TiLPInputBuffer 
			EndIf
		EndIf		
		
		High 0
		
		Let B3 = 0
		Do While B3 != HSerPtr
		   Get B3, B0
			#IfDef Debug
				SerTxd("Out", 9, B0, 9, #B0, CR, LF)
			#EndIf
			GoSub TiLPOut
			Inc B3
		Loop
		
		HSerPtr = 0
		
		Low 0
		
	Loop
	
}


; ================================================================= ;
; Texas Instruments Link Protocol routines.                         ;
; ================================================================= ;
{

; ----------------------------------------------------------------- ;
; TiLPIn - Input a byte via the Texas Instruments link protocol.    ;
; ----------------------------------------------------------------- ;
; Inputs:   None.                                                   ;
; Outputs:  B0 = Byte input.                                        ;
; Destroys: B1, B2.                                                 ;
; ----------------------------------------------------------------- ;
TiLPIn:
	; Shift in eight bits:
	For B1 = 0 To 7
	
		; Shift >>= 1
	    Let B0 = B0 / 2
	    
	    ; Wait for either pin to go low:
	    Let B2 = 10
	    Do
	    	If Pin0 = 0 Then TiLPInBit0
	    	If Pin1 = 0 Then TiLPInBit1
	    	Dec B2
	    	If B2 = 0 Then Return : EndIf
	    Loop
	    
	    ; Received a 0 bit:
	    TiLPInBit0:
	    	Low PortC 1	    	        ; Hold data 1 low.
	    	If Pin0 = 0 Then TiLPInBit0 ; Wait for data 0 to go high.
	    	Let DirsC = %00000000       ; Release data 1.
			Goto GetNextBit
		
		; Received a 1 bit:
	    TiLPInBit1:
	    	Low PortC 0                 ; Hold data 0 low.
	    	Let Bit7 = 1                ; Add the bit to B0.
	    	If Pin1 = 0 Then TiLPInBit1 ; Wait for data 1 to go high.
	    	Let DirsC = %00000000 	    ; Release data 0.
	    
	GetNextBit:
	Next B1
	Return

; ----------------------------------------------------------------- ;
; TiLPOut - Output a byte via the Texas Instruments link protocol.  ;
; ----------------------------------------------------------------- ;
; Inputs:   B0 = Byte to output.                                    ;
; Outputs:  None.                                                   ;
; Destroys: B1.                                                     ;
; ----------------------------------------------------------------- ;
TiLPOut:
	For B1 = 0 To 7
		If Bit0 = 0 Then
			Low PortC 0
			Do While Pin1 = 1 : Loop
			Let DirsC = %00000000
			Do While Pin1 = 0 : Loop
		Else			
			Low PortC 1
			Do While Pin0 = 1 : Loop
			Let DirsC = %00000000
			Do While Pin0 = 0 : Loop
		EndIf
	    Let B0 = B0 / 2
	Next B1
	Return
	
}










