.module TiLP

Neutral = %11010000
TimeOut = 50

; ---------------------------------------------------------
; ReadByteFast -> Read a byte using the TI link protocol.
; ---------------------------------------------------------
; Inputs:    None.
; Outputs:   z = byte received; nz = no byte received.
;            a = byte (if received).
; Destroyed: af.
; Remarks:   Exits instantly if a byte isn't ready.
; ---------------------------------------------------------
ReadByteFast
	in a,(bport)
	and %11
	cp %11
	jr nz,ReadByte
	inc a
	ret

; ---------------------------------------------------------
; ReadByte -> Read a byte using the TI link protocol.
; ---------------------------------------------------------
; Inputs:    None.
; Outputs:   z = byte received; nz = no byte received.
;            a = byte (if received).
; Destroyed: af.
; ---------------------------------------------------------
ReadByte
	push hl \ push de \ push bc
	call ReadByteNoPreserve
	ld a,c
	pop bc \ pop de \ pop hl
	ret
	

	
; ---------------------------------------------------------
; ReadByteNoPreserve -> Read a byte.
; ---------------------------------------------------------
; Inputs:    None.
; Outputs:   z = byte received; nz = no byte received.
;            c = byte (if received).
; Destroyed: af, bc, de, l.
; ---------------------------------------------------------
ReadByteNoPreserve
	ld d,8
ReadBit

	ld l,TimeOut
--	ld b,0
-	in a,(bport)
	and %11
	cp %11
	jr nz,{+}
	djnz {-}
	dec l
	jr nz,{--}
	inc b
	ret
+

	rrca
	rr c
	rlca

	or Neutral
	out (bport),a

	and %11
	xor %11
	ld e,a

	ld l,TimeOut
--	ld b,0
-	in a,(bport)
	and e
	jr nz,{+}
	djnz {-}
	dec l
	jr nz,{--}
	inc b
	ret

+	; Bring both lines back up
	ld a,Neutral
	out (bport),a

	ld l,TimeOut
--	ld b,0
-	in a,(bport)
	and %11
	cp %11
	jr z,{+}
	djnz {-}
	dec l
	jr nz,{--}
	inc b
	ret
+
	dec d
	ret z
	jr ReadBit


; ---------------------------------------------------------
; WriteByte -> Send a byte using the TI link protocol.
; ---------------------------------------------------------
; Inputs:    a = byte to send.
; Outputs:   z = byte sent; nz = failed.
; Destroyed: af.
; ---------------------------------------------------------
WriteByte
	push hl \ push de \ push bc
	ld c,a
	call WriteByteNoPreserve
	pop bc \ pop de \ pop hl
	ret
	
; ---------------------------------------------------------
; WriteByteNoPreserve -> Send a byte.
; ---------------------------------------------------------
; Inputs:    c = byte to send.
; Outputs:   z = byte sent; nz = failed.
; Destroyed: af, bc, de, hl.
; ---------------------------------------------------------
WriteByteNoPreserve
	ld d,8
WriteBit
	ld a,Neutral >> 1
	rrc c
	rla
	inc a
	out (bport),a
	
	ld e,TimeOut
--	ld b,0
-	in a,(bport)
	and %11
	jr z,{+}
	djnz {-}
	dec e
	jr nz,{--}
	inc b
	ret
+
	
	ld a,Neutral
	out (bport),a
	
	ld e,TimeOut
--	ld b,0
-	in a,(bport)
	and %11
	cp %11
	jr z,{+}
	djnz {-}
	dec e
	jr nz,{--}
	inc b
	ret
+
	dec d
	ret z
	jr WriteBit

; ---------------------------------------------------------
; WriteString -> Send a string.
; ---------------------------------------------------------
; Inputs:    hl -> NUL-terminated string to send.
; Outputs:   z = string sent; nz = failed.
;            hl -> next string (after NUL byte).
; Destroyed: af, hl.
; ---------------------------------------------------------
WriteString
	push bc
	push de
	
-	ld a,(hl)
	inc hl
	or a
	jr z,Finished
	ld c,a
	push hl
	call WriteByteNoPreserve
	pop hl
	jr z,{-}
	
Finished
	pop de
	pop bc
	ret

; ---------------------------------------------------------
; WriteStringOP1 -> Send a string in OP1.
; ---------------------------------------------------------
; Inputs:    OP1 = NUL-terminated string to send.
; Outputs:   z = string sent; nz = failed.
; Destroyed: af.
; ---------------------------------------------------------
WriteStringOP1
	push hl
	ld hl,OP1
	call WriteString
	pop hl
	ret

.endmodule
