; =========================================================
; Module: VMusic2
; =========================================================
; Description:
;     VMusic2-related functions.
; =========================================================

.module VMusic2 \ VMusic2

.var ubyte[256], InputBuffer
.var uword, InputBufferPointer
.var ubyte, LastEvent

.include "Modules/TiLP.asm"	

.include "Modules/Utility.asm"

.include "Modules/Directory.asm"
.include "Modules/Disk.asm"
.include "Modules/Music.asm"

.module Events
	None = 0
	.module Directory
		Listing        = $01
		Change         = $02
	.endmodule
	.module File
		Size           = $03
	.endmodule
	.module Disk
		FreeSpace      = $12
	.endmodule
	.module Music
		Play           = $1D
		Stop           = $20
		Next           = $25
		Previous       = $26
		PlayAll        = $21
		WriteRegister  = $1E
		ReadRegister   = $1F
	.endmodule
.endmodule

; ---------------------------------------------------------
; Initialise -> Process VMusic2 updates and trigger events.
; ---------------------------------------------------------
; Destroys: af, bc, de, hl.
; ---------------------------------------------------------
Initialise
	ld hl,InputBuffer
	ld (InputBufferPointer),hl
	xor a
	ld (LastEvent),a
	ret

; ---------------------------------------------------------
; Update -> Process VMusic2 updates and trigger events.
; ---------------------------------------------------------
; Destroys: af, bc, de, hl.
; ---------------------------------------------------------
Update
	call TiLP.ReadByteFast
	ret nz

	ld hl,(InputBufferPointer)
-	ld (hl),a
	cp '\r'
	jr z,EndOfCommand
	inc hl
	ld (InputBufferPointer),hl
UpdateAgain
	call TiLP.ReadByte
	jr z,{-}
	ret
	
EndOfCommand
	ld (hl),0
	ld hl,InputBuffer


	ld a,(LastEvent)
	ld e,a

	call GetInputType	
	ld d,a
	or a
	jr z,{+}
	xor a
	ld (LastEvent),a
+	

	push de
	push hl
	
	; Event handler is passed parameters:
	; hl  -> input string.
	; d   =  type (0 = data).
	; a,e =  event method.
	ld a,e
	call Parent.VMusic2EventHandler	
	pop hl
	pop de

	ld (InputBufferPointer),hl
	ret
	jr UpdateAgain

; ---------------------------------------------------------
; GetIsFree -> Check if the device is free. 
; ---------------------------------------------------------
; Inputs:   None.
; Outputs:  z = device is free; nz = device is busy.
; Destroys: af.
; ---------------------------------------------------------
GetIsFree
	ld a,(LastEvent)
	or a
	ret

; ---------------------------------------------------------
; CancelIfBusy -> Cancel a method call if device is busy.
; ---------------------------------------------------------
; Inputs:   None.
; Outputs:  If device is free: z, routine continues.
;           If device is busy: nz, routine is cancelled.
; Destroys: af.
; Remarks:  Intended for internal usage only.
; ---------------------------------------------------------
CancelIfBusy
	call GetIsFree
	ret z
	pop hl
	ret

; ---------------------------------------------------------
; SetUpEvent -> Send a command and set up for what to do
;               when data comes back from the VMusic2.
; ---------------------------------------------------------
; Inputs:   hl -> NUL-terminated command string.
;           a  =  event type.
; Outputs:  z = set up successfully; nz = failed.
; Destroys: af, hl.
; Remarks:  Intended for internal usage only.
; ---------------------------------------------------------
SetUpEvent
	push de
	ld de,ArgsNone
	ld (SetUpEventArgs),de
	pop de
	jp ExecCommand
	
; ---------------------------------------------------------
; SetUpEventOP1Argument -> Send a command with OP1 as an
;                          argument.
; ---------------------------------------------------------
; Inputs:   hl -> NUL-terminated command string.
;           a  =  event type.
;           OP1 = NUL-terminated argument.
; Outputs:  z = set up successfully; nz = failed.
; Destroys: af, hl.
; Remarks:  Intended for internal usage only.
; ---------------------------------------------------------
SetUpEventOP1Argument
	push de
	ld de,ArgsOP1NulTerminated
	ld (SetUpEventArgs),de
	pop de
	jp ExecCommand
	
SetUpEventOP1LengthPrefixedArgument
	push de
	ld de,ArgsOP1LengthPrefixed
	ld (SetUpEventArgs),de
	pop de
	jp ExecCommand

; INTERNAL
ExecCommand
	push hl
	ld l,a
	ld a,(LastEvent)
	or a
	ld a,l
	pop hl
	ret nz

	cp a
	push af
	call TiLP.WriteString
	jr nz,EventArgsFail
	
SetUpEventArgs = $+1
	jp $0000
	
EventArgsSuccess	
	ld a,'\r'
	call TiLP.WriteByte
	jr nz,EventArgsFail
	
	pop af
	
	ld (LastEvent),a
	ld hl,InputBuffer
	ld (InputBufferPointer),hl
	ret
	
EventArgsFail
	pop hl
	ret
	
; INTERNAL
ArgsNone
	jp EventArgsSuccess 
; INTERNAL
ArgsOP1NulTerminated
	ld a,' '
	call TiLP.WriteByte
	jp nz,EventArgsFail
	ld hl,OP1
	call TiLP.WriteString
	jp nz,EventArgsFail
	jp EventArgsSuccess
; INTERNAL
ArgsOP1LengthPrefixed
	ld a,' '
	call TiLP.WriteByte
	jp nz,EventArgsFail
	ld hl,OP1
	ld b,(hl)
	inc hl
-	ld c,(hl)
	inc hl
	push hl
	push bc
	call TiLP.WriteByteNoPreserve
	pop bc
	pop hl
	jp nz,EventArgsFail
	djnz {-}
	jp EventArgsSuccess

; ---------------------------------------------------------
; GetInputType -> Get the type of input.
; ---------------------------------------------------------
; Inputs:   None.
; Outputs:  a = type:
;               0 = Data.
;               1 = Prompt.
;               2 = Bad Command.
;               3 = Command Failed.
;           z = is data; nz is not data.
; Destroys: f.
; ---------------------------------------------------------
GetInputType
	push hl
	push de
	push bc
	
	ld c,3
	
	ld hl,InputBuffer
	ld de,CommandFailed
	ld b,CommandFailedSize
	call StrCmp
	jr z,{+}
	dec c

	ld hl,InputBuffer
	ld de,BadCommand
	ld b,BadCommandSize
	call StrCmp
	jr z,{+}
	dec c

	ld hl,InputBuffer
	ld de,Prompt
	ld b,PromptSize
	call StrCmp
	jr z,{+}
	dec c
	
+	ld a,c
	or a
	pop bc
	pop de
	pop hl
	ret
Prompt        .db 'D:\\>', 0          \ PromptSize        = $-Prompt
BadCommand    .db 'Bad Command', 0    \ BadCommandSize    = $-BadCommand
CommandFailed .db 'Command Failed', 0 \ CommandFailedSize = $-CommandFailed

; ---------------------------------------------------------
; StrCmp -> Compare two strings.
; ---------------------------------------------------------
; Inputs:   hl -> string 1
;           de -> string 2
;           b  =  string length.
; Outputs:  z = same, nz = different.
; Destroys: af, b, de, hl.
; ---------------------------------------------------------
StrCmp
	ld a,(de)
	cp (hl)
	ret nz
	inc hl
	inc de
	djnz StrCmp
	ret

; ---------------------------------------------------------
; CallFromTable -> Call a method based on a table.
; ---------------------------------------------------------
; Inputs:   ix -> table.
;           a  =  method id.
; Outputs:  None.
; Destroys: ix and f are destroyed entering the event; all
;           other registers can be destroyed by the method
;           being called.
; Remarks:  Table format: [db event id][dw handler address]
;           Last item must have an event id of 255.
; ---------------------------------------------------------
CallFromTable
	cp (ix+0)
	jr z,{+}
	inc (ix+0)
	jr z,EndOfTable
	inc ix \ inc ix \ inc ix
	jr CallFromTable

+	ld (RestoreHL),hl
	ld l,(ix+1)
	ld h,(ix+2)
	push hl
RestoreHL = $+1
	ld hl,$0000
	ret

EndOfTable
	dec (ix+0)
	ret

.echoln "VMusic2 - Ben Ryves for MaxCoderz 2007 ({0} bytes).", $-VMusic2

.endmodule